/*
Module : CPL_PP.CPP
Purpose: Implementation for an MFC wrapper class for Control Panel Applets
Created: PJN / 18-07-1999
History: PJN / 20-11-1999 1. Now fully supports CPL_INQUIRE aswell as CPL_NEWINQUIRE messages. 
                          2. Now supports a document template approach to creating applets. This means that 
                          you can easily support multiple applets per dll using the CPL++ framework. 
                          3. Demo program now supports demo'ing both static and dynamic applets. Control panel 
                          applet now always output as a ".cpl" file instead of a ".dll" file. 
                          4. Now supports displaying the specified property page (via the CPL_STARTWPARMS 
                          message) when a control panel applet is called via the rundll32 command line.
         PJN / 08-08-2002 1. Included copyright message in source code.
                          2. Now includes framework code to register and unregister the applet as per new
                          Windows 2000 coding guidelines. See the MSDN for further details. This basically
                          means that you do not need to copy your applet into the Windows System directory
                          for it to be recognized by the OS. Thanks to Armen Hakobyan for this suggestion.
                          3. Updated the sample applet to use standard COM registration methods of 
                          DllRegisterServer and DllUnregisterServer. This allows your applets to be self-
                          registering thro standard apps such as regsvr32.
                          4. Updated demo applet to remove non-existent resources.
         PJN / 02-10-2002 1. Fixed small typo in the help file. Thanks to "Marty" for spotting this.
                          2. Implemented DECLARE_CPLAPP / IMPLEMENT_CPLAPP and DECLARE_CPLREGISTER macros. 
                          This keeps client code as clean as possible. Basically something similiar to the 
                          MFC / ATL macros. 
                          3. Made the CPlApplet callback function visible in the public names space again 
                          through the use of the DECLARE_CPLAPP macro. This means that the class framework 
                          can be safely used in an MFC extension DLL. Thanks to Armen Hakobyan for the last 
                          2 great updates.
                          4. Fixed an issue / bug in the registration process methods where the Applet name 
                          had spaces in it. For some reason the Control Panel does not like this. The code 
                          now registers using a name which has "_" in place of spaces.
         PJN / 21-10-2002 1. Now supports registration of a Category. This is a new feature in Windows XP
                          and Windows Server 2003 where control panel applets can be grouped into a
                          category. For more info on the available categories, please see the MS KB 
                          article Q292463. Thanks to Pyklmki Juha for suggesting this improvement.
         PJN / 04-05-2003 1. Creation of CDialog based applets now correctly observe the parent window as
                          passed into it. Thanks to Vijay Chegu for suggesting the fix for this.
         PJN / 18-03-2004 1. Updated the sample app settings, to support debugging the sample applet. Please
                          note that if you do not have Windows installed to C:\Windows then you will need
                          to change your project's Debug settings to point to your Windows installation 
                          directory. Unfortunately I was unable to find a way to handle this automatically.
                          2. Fixed an issue with the support for CPL_STARTWPARAMS on XP. Thanks to Jon
                          Baggott for researching this area and providing the fix.
                          3. The Framework and sample applet now correctly sets up support for XP Visual Styles.
                          Because of this you will need to have a recentish copy of the Platform SDK installed
                          to get the correct defines for "RT_MANIFEST".
                          4. Moved framework and sample app up to VC 6.
         PJN / 23-07-2004 1. Now uses relative paths for debugging settings. This means that so long as you 
                          install the sample app on your system drive then debugging of the sample app will
                          work out of the box.
                          2. In CControlPanelApp::RegisterAppletViaRegistry(), Fixed an issue where the Cpls
                          regkey doesn't always exist and registration via regsvr32 fails with return
                          code 80020007 (if memory serves), which is a file not found error.  This can
                          be fixed by changing the call to RegOpenKeyEx to RegCreateKeyEx. Thanks to Jason 
                          Slater for reporting this issue. This change is also applied to the function
                          CControlPanelApp::RegisterCategory which registers a Control Panel Applet's
                          Category.
         PJN / 20-10-2004 1. Fixed a number of compiler warnings when the Detect 64 bit compatibility issues
                          option is enabled in Visual Studio .Net.
         PJN / 07-07-2006 1. Updated copyright details.
                          2. Code now uses newer C++ style casts instead of C style casts.
                          3. Updated documentation to use the same style as the web site.
                          4. Replaced calls to ZeroMemory with memset.
                          5. Removed various unused destructors from the sample applet provided with the code.
                          6. Updated the code to clean compile on VC 2005
                          7. Code now uses SEVERITY_ERROR define for MAKE_HRESULT calls.
                          8. Updated the definition of all the virtual CControlPanelApp and CControlPanelApplet 
                          functions to use the same layout as defined by the MSDN. If you are upgrading your 
                          code, please ensure you review your code to match the new layout.
         PJN / 05-08-2006 1. Fix for some compiler errors when the code is compiled for Win64.
         PJN / 12-04-2009 1. Updated copyright details.
                          2. Removed VC 6 style classwizard comments from the code.
                          3. Updated code to compile correctly using _ATL_CSTRING_EXPLICIT_CONSTRUCTORS define
                          4. The code now only supports VC 2005 or later.
                          5. Code now compiles cleanly using Code Analysis (/analyze)
                          6. Updated code to clean compile on VC 2008.
                          7. Now supports Vista specific registration using a GUID. This is required to support
                          task links being associated with a Control Panel item.
                          8. RegisterCategory now supports per user registration. It has also been renamed
                          to RegisterXPCategory
                          9. Now supports Vista specific category registration via RegisterVistaCategory. 
                          10. Now supports Vista multi category registration via RegisterVistaCategories 
                          11. Now supports Vista specific canonical name registration via RegisterCanonicalName
                          12. Now supports Vista specific task link registration via RegisterTaskLinks
                          13. RegisterApplet now has support for replacable parameters. The two supported values
                          are %MODULE% which is the full path to the current DLL and %APPPATH% which represents
                          the directory associated with %MODULE%.
                          14. Now supports Vista specific safe mode registration via RegisterSafeMode
                          15. Now supports XP multi category registration via RegisterXPCategories 
         PJN / 07-09-2009 1. Fixed a bug in the declaration of IMPLEMENT_DYNAMIC(CControlPanelApp...). It was
                          incorrectly setting the 2nd parameter to also be "CControlPanelApp". This was uncovered
                          due to a change in the implementation of CDialog::DoModal in VC 2005. Thanks to Ivo
                          Rytir for reporting this bug.
                          

Copyright (c) 1999 - 2009 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////// Includes /////////////////////////////////////////////////////

#include "stdafx.h"
#include "cpl_pp.h"


//////////////// Macros / Defines /////////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


//////////////// Implementation ///////////////////////////////////////////////

CControlPanelApp* GetControlPanelApp()
{
  CControlPanelApp* pApp = static_cast<CControlPanelApp*>(AfxGetApp());
  AFXASSUME(pApp);
  ASSERT(pApp->IsKindOf(RUNTIME_CLASS(CControlPanelApp)));
  return pApp;
}

//Class which allows us to modify CDialog::m_pParentWnd
class CCPLDialog : public CDialog
{
public:
//Methods
  void SetParentWnd(CWnd* pParentWnd) 
  { 
    m_pParentWnd = pParentWnd; 
  };
};

IMPLEMENT_DYNAMIC(CControlPanelApplet, CCmdTarget);

CControlPanelApplet::CControlPanelApplet(UINT nResourceID, CRuntimeClass* pUIClass)
{
  //Validate our parameters
  AFXASSUME(pUIClass);
  ASSERT(pUIClass->IsDerivedFrom(RUNTIME_CLASS(CDialog)) || 
         pUIClass->IsDerivedFrom(RUNTIME_CLASS(CPropertySheet))); //The runtime class must be derived from one of these classes
         
  //Store the parameters away in member variables
  m_nResourceID = nResourceID;
  m_pUIClass = pUIClass;
  m_bDynamic = TRUE;
  m_nDescriptionID = 0;
  m_nPageNumber = 0;

  //Store away the name of the applet for later use
  CString sResource;
  VERIFY(sResource.LoadString(m_nResourceID)); //Did you forget to specify a string resource for the applet?
  AfxExtractSubString(m_sName, sResource, 0);
}

CControlPanelApplet::CControlPanelApplet(UINT nResourceID, UINT nDescriptionID, CRuntimeClass* pUIClass)
{
  //Validate our parameters
  AFXASSUME(pUIClass);
  ASSERT(pUIClass->IsDerivedFrom(RUNTIME_CLASS(CDialog)) || 
         pUIClass->IsDerivedFrom(RUNTIME_CLASS(CPropertySheet))); //The runtime class must be derived from one of these classes

  //Store the parameters away in member variables
  m_nResourceID = nResourceID;
  m_nDescriptionID = nDescriptionID;
  m_pUIClass = pUIClass;
  m_bDynamic = FALSE;
  m_nPageNumber = 0;

  //Store away the name of the applet for later use
  VERIFY(m_sName.LoadString(nResourceID));
}

LONG CControlPanelApplet::OnStartParms(CWnd* pParentWnd, LPCTSTR lpszExtra)
{
  if (lpszExtra)
    m_nPageNumber = ::_ttoi(lpszExtra);
  
  if (m_nPageNumber != 0)
    --m_nPageNumber; //We're 0 based

  return !OnRun(pParentWnd);
}

LONG CControlPanelApplet::OnRun(CWnd* pParentWnd)
{
  CWnd* pWnd = static_cast<CWnd*>(m_pUIClass->CreateObject()); 
  LONG nResult = 1; //Assume failure
  if (pWnd)
  {
    nResult = 0; //Success;
    if (pWnd->IsKindOf(RUNTIME_CLASS(CPropertySheet)))
    {
      CPropertySheet* pSheet = static_cast<CPropertySheet*>(pWnd);
      pSheet->Construct(m_sName, pParentWnd, m_nPageNumber);

      pSheet->DoModal(); //Display the sheet
    }
    else
    {
      ASSERT(pWnd->IsKindOf(RUNTIME_CLASS(CDialog)));
      CCPLDialog* pDialog = static_cast<CCPLDialog*>(pWnd);
      pDialog->SetParentWnd(pParentWnd); //Set the parent window

      pDialog->DoModal(); //Display the dialog
    }

    //Don't forget to tidy up the memory used
    delete pWnd;
  }

  return nResult;
}

LONG CControlPanelApplet::OnInquire(CPLINFO* pInfo)
{
  //validate our parameters
  AFXASSUME(pInfo);

  pInfo->lData  = reinterpret_cast<LONG_PTR>(this);

  if (m_bDynamic)
  {
    pInfo->idIcon = CPL_DYNAMIC_RES;
    pInfo->idName = CPL_DYNAMIC_RES;
    pInfo->idInfo = CPL_DYNAMIC_RES;
  }
  else
  {
    pInfo->idIcon = m_nResourceID;    //The resource id of the icon to use
    pInfo->idName = m_nResourceID;    //The resource id of the name
    pInfo->idInfo = m_nDescriptionID; //The resource id of the description
  }

  return 0; //Message was handled
}

LONG CControlPanelApplet::OnNewInquire(NEWCPLINFO* pInfo)
{
  //validate our parameters
  AFXASSUME(pInfo);

  if (!m_bDynamic)
    return 1; //Fail it, we want OnInquire instead

  pInfo->dwSize = sizeof(NEWCPLINFO);
  pInfo->dwFlags = 0;
  pInfo->dwHelpContext = 0;
  pInfo->lData = reinterpret_cast<LONG_PTR>(this);

  //Load the icon from the icon resource with the same ID as the string
  //resource "m_nResourceID". This is similar to the way MFC implements document templates
  pInfo->hIcon = GetControlPanelApp()->LoadIcon(m_nResourceID);
  ASSERT(pInfo->hIcon != NULL);  //Did you forget to specify an icon for this applet

  //Load the resource string. The format is "Name\nDescription\nHELPFILE.HLP"
  CString sResource;
  VERIFY(sResource.LoadString(m_nResourceID)); //Did you forget to specify a string resource for the applet?

  //Split the string into the 3 parts
  ASSERT(m_sName.GetLength());      //Name should have been loaded
  ASSERT(m_sName.GetLength() < sizeof(pInfo->szName)/sizeof(TCHAR));
  _tcscpy_s(pInfo->szName, sizeof(pInfo->szName)/sizeof(TCHAR), m_sName);

  CString sDescription;
  AfxExtractSubString(sDescription, sResource, 1);
  ASSERT(sDescription.GetLength()); //Description should have been loaded
  ASSERT(sDescription.GetLength() < sizeof(pInfo->szInfo)/sizeof(TCHAR));
  _tcscpy_s(pInfo->szInfo, sizeof(pInfo->szInfo)/sizeof(TCHAR), sDescription);

  CString sHelpFile;
  AfxExtractSubString(sHelpFile, sResource, 2);
  if (sHelpFile.GetLength())
  {
    ASSERT(sHelpFile.GetLength() < sizeof(pInfo->szHelpFile)/sizeof(TCHAR));
    _tcscpy_s(pInfo->szHelpFile, sizeof(pInfo->szHelpFile)/sizeof(TCHAR), sHelpFile);
  }
  else
  {
    _tcscpy_s(pInfo->szHelpFile, sizeof(pInfo->szHelpFile)/sizeof(TCHAR), _T(""));
  }

  return 0; //message was handled
}

LONG CControlPanelApplet::OnSelect()
{
  return 0; //Default is do nothing for CPL_SELECT
}

LONG CControlPanelApplet::OnStop()
{
  return 0; //Default is do nothing. You could if you want
            //post a WM_CLOSE to yourself or do a EndDialog
            //if you wanted to close your applet when the 
            //control panel exits.
}



IMPLEMENT_DYNAMIC(CControlPanelApp, CWinApp);

CControlPanelApp::CControlPanelApp()
{
  //Doesn't need to do anything
}

CControlPanelApp::~CControlPanelApp()
{
  //Free up the memory for all the applets
  while (!m_Applets.IsEmpty())
    delete m_Applets.RemoveHead();
}

void CControlPanelApp::AddApplet(CControlPanelApplet* pApplet)
{
  ASSERT(pApplet);
  m_Applets.AddTail(pApplet);
}

LONG CControlPanelApp::OnInit()
{
  //This is the function you should call AddApplet 
  //in, in your derived class
  LONG nApplets = static_cast<LONG>(m_Applets.GetCount());
  if (nApplets == 0)
  {
    ASSERT(FALSE); //You've forget to call AddApplet in your version of OnInit
                   //before calling this versions
  }

  InitCommonControls(); //Required for XP Visual Styles support

  return nApplets;
}

LONG CControlPanelApp::OnExit()
{
  //Default is do nothing for CPL_EXIT
  return 1; //Success
}

LONG CControlPanelApp::OnCplMsg(HWND hWndCPl, UINT uMsg, LPARAM lParam1, LPARAM lParam2)
{
  LONG nResult = 1;
  if (uMsg == CPL_INIT)
    nResult = OnInit();
  else if (uMsg == CPL_EXIT)
    nResult = OnExit();
  else if (uMsg == CPL_GETCOUNT)
  {      
    nResult = static_cast<LONG>(m_Applets.GetCount());
    if (nResult == 0)
      ASSERT(FALSE); //You've forget to call AddApplet, You should do it in you apps OnInit
  }
  else
  {
    //It's a message destined for a specific applet, Find the specific one
    //and dispatch the message to it
    POSITION posApplet = m_Applets.FindIndex(lParam1);
    ASSERT(posApplet); 
    CControlPanelApplet* pApplet = m_Applets.GetAt(posApplet);  
    AFXASSUME(pApplet);

    switch (uMsg)
    {
      case CPL_INQUIRE:
      {
        CPLINFO* pInfo = reinterpret_cast<LPCPLINFO>(lParam2);
        nResult = pApplet->OnInquire(pInfo);
        break;
      }  
      case CPL_NEWINQUIRE:
      {
        NEWCPLINFO* pInfo = reinterpret_cast<LPNEWCPLINFO>(lParam2);
        nResult = pApplet->OnNewInquire(pInfo);
        break;
      }  
#ifdef _UNICODE
      case CPL_STARTWPARMSA:
#else
      case CPL_STARTWPARMSW:
#endif
      {
        nResult = 0;
        break;
      }
      case CPL_STARTWPARMS:
      {
        CWnd* pParentWnd = CWnd::FromHandle(hWndCPl);
        LPCTSTR lpszExtra = reinterpret_cast<LPCTSTR>(lParam2);
        nResult = pApplet->OnStartParms(pParentWnd, lpszExtra);
        break;
      }
      case CPL_DBLCLK:
      {
        CWnd* pParentWnd = CWnd::FromHandle(hWndCPl);
        nResult = pApplet->OnRun(pParentWnd);
        break;
      }
      case CPL_SELECT:
      {
        nResult = pApplet->OnSelect();
        break;
      }
      case CPL_STOP:
      {
        nResult = pApplet->OnStop();
        break;
      }
      default:
      {
        TRACE(_T("CControlPanelApp::OnCplMsg, Received an unknown control panel message:%d\n"), uMsg);
        nResult = 1;
        break;
      }
    }
  }

  return nResult;
}

BOOL CControlPanelApp::RegisterApplet(LPCTSTR pszAppPath, BOOL bRegister, BOOL bForAllUsers, LPCTSTR pszCategory, LPCTSTR pszGUID, LPCTSTR pszCanonicalName, LPCTSTR pszTaskLinksFile, SafeModeValue safeModeValue)
{
  //Delegate the work to the other version of the method after doing the replacement of string parameters
  return RegisterApplet(DoReplaceableStringParameters(pszAppPath), m_pszAppName, bRegister, bForAllUsers, pszCategory, pszGUID, pszCanonicalName, DoReplaceableStringParameters(pszTaskLinksFile), safeModeValue);
}

BOOL CControlPanelApp::RegisterApplet(LPCTSTR pszAppPath, LPCTSTR pszAppName, BOOL bRegister, BOOL bForAllUsers, LPCTSTR pszCategory, LPCTSTR pszGUID, LPCTSTR pszCanonicalName, LPCTSTR pszTaskLinksFile, SafeModeValue safeModeValue)
{ 
  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;        

  //Get the version info and from that the various flags which control how registration is done
  OSVERSIONINFO osvi;
  memset(&osvi, 0, sizeof(OSVERSIONINFO));
  osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  if (!GetVersionEx(&osvi))
    return FALSE;
 
  BOOL bRegistryRegistration = ((osvi.dwPlatformId == VER_PLATFORM_WIN32_NT && osvi.dwMajorVersion >= 5 ) || (osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS && (osvi.dwMinorVersion >= 90 || osvi.dwMajorVersion > 4)));
  BOOL bVista = ((osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) && (osvi.dwMajorVersion >= 6));
  BOOL bXP = ((osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) && ((osvi.dwMajorVersion > 5 ) || ((osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion >= 1))));
    
  //Hand off to the correct helper function
  if (bRegistryRegistration)
  {
    bSuccess = RegisterAppletViaRegistry(bRegister, bForAllUsers, pszAppName, pszAppPath);
    if (bSuccess)
    {
      //Register our Category/Categories if necessary
      if (_tcslen(pszCategory))
      {
        //Determine if we are registering for multiple categories or a single category
        BOOL bSingleCategory = (_tcsstr(pszCategory, _T(",")) == NULL);
        if (bSingleCategory)
        {
          //Obtain the DWORD version of the category
          DWORD dwCategory = _ttoi(pszCategory);
        
          //Determine if we should registry the category using Vista style or XP style registration
          if (bVista)
            bSuccess = RegisterVistaCategory(bRegister, bForAllUsers, dwCategory, pszAppPath);
          else if (bXP)
            bSuccess = RegisterXPCategory(bRegister, bForAllUsers, dwCategory, pszAppPath);
        }
        else
        {
          //Determine if we should registry the category using Vista style or XP style registration
          if (bVista)
            bSuccess = RegisterVistaCategories(bRegister, bForAllUsers, pszCategory, pszAppPath);
          else if (bXP)
            bSuccess = RegisterXPCategories(bRegister, bForAllUsers, pszCategory, pszAppPath);
        }
      }
      
      if (bVista)
      {
        //Register our GUID also if necessary
        if (bSuccess && _tcslen(pszGUID))
          bSuccess = RegisterGUID(bRegister, bForAllUsers, pszGUID, pszAppPath);

        //Register our Canonical Name also if necessary
        if (bSuccess && _tcslen(pszCanonicalName))
          bSuccess = RegisterCanonicalName(bRegister, bForAllUsers, pszCanonicalName, pszAppPath);

        //Register our tasks links if necessary
        if (bSuccess && _tcslen(pszTaskLinksFile))
          bSuccess = RegisterTaskLinks(bRegister, bForAllUsers, pszTaskLinksFile, pszAppPath);
        
        //Register our safe mode setting if necessary
        if (bSuccess && safeModeValue != NotSpecified)
          bSuccess = RegisterSafeMode(bRegister, bForAllUsers, static_cast<DWORD>(safeModeValue), pszAppPath);
      }
    }
  }
  else    
    bSuccess = RegisterAppletViaIniFile(bRegister, pszAppName, pszAppPath);    

  return bSuccess;
}

BOOL CControlPanelApp::RegisterAppletViaIniFile(BOOL bRegister, LPCTSTR pszAppName, LPCTSTR pszAppPath)
{
  //Avoid any spaces in the name with underscores. Using spaces in the name
  //causes errors when the cpl is invoked
  CString sAppName(pszAppName);
  sAppName.Replace(_T(' '), _T('_'));

  return WritePrivateProfileString(_T("MMCPL"), sAppName, (bRegister ? pszAppPath : NULL), _T("control.ini"));    
}

BOOL CControlPanelApp::RegisterAppletViaRegistry(BOOL bRegister, BOOL bForAllUsers, LPCTSTR pszAppName, LPCTSTR pszAppPath)
{
  //Avoid any spaces in the name with underscores. Using spaces in the name
  //causes errors when the cpl is invoked
  CString sAppName(pszAppName);
  sAppName.Replace(_T(' '), _T('_'));

  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Cpls"), 0, NULL, 
                                 REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, sAppName, 0, REG_SZ, reinterpret_cast<const BYTE*>(pszAppPath), static_cast<DWORD>((_tcslen(pszAppPath) + 1) * sizeof(TCHAR)));
    else
      nSuccess = RegDeleteValue(hKey, sAppName);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

BOOL CControlPanelApp::RegisterXPCategory(BOOL bRegister, BOOL bForAllUsers, DWORD dwCategoryID, LPCTSTR pszAppPath)
{
  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Extended Properties\\{305CA226-D286-468e-B848-2B2E8E697B74} 2"), 
                                 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, pszAppPath, 0, REG_DWORD, reinterpret_cast<const BYTE*>(&dwCategoryID), sizeof(dwCategoryID));
    else
      nSuccess = RegDeleteValue(hKey, pszAppPath);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

BOOL CControlPanelApp::RegisterXPCategories(BOOL bRegister, BOOL bForAllUsers, LPCTSTR pszCategories, LPCTSTR pszAppPath)
{
  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Extended Properties\\{305CA226-D286-468e-B848-2B2E8E697B74} 2"), 
                                 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, pszAppPath, 0, REG_SZ, reinterpret_cast<const BYTE*>(pszCategories), static_cast<DWORD>((_tcslen(pszCategories) + 1) * sizeof(TCHAR)));
    else
      nSuccess = RegDeleteValue(hKey, pszAppPath);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

BOOL CControlPanelApp::RegisterVistaCategory(BOOL bRegister, BOOL bForAllUsers, DWORD dwCategoryID, LPCTSTR pszAppPath)
{
  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Extended Properties\\System.ControlPanel.Category"), 
                                 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, pszAppPath, 0, REG_DWORD, reinterpret_cast<const BYTE*>(&dwCategoryID), sizeof(dwCategoryID));
    else
      nSuccess = RegDeleteValue(hKey, pszAppPath);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

BOOL CControlPanelApp::RegisterVistaCategories(BOOL bRegister, BOOL bForAllUsers, LPCTSTR pszCategories, LPCTSTR pszAppPath)
{
  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Extended Properties\\System.ControlPanel.Category"), 
                                 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, pszAppPath, 0, REG_SZ, reinterpret_cast<const BYTE*>(pszCategories), static_cast<DWORD>((_tcslen(pszCategories) + 1) * sizeof(TCHAR)));
    else
      nSuccess = RegDeleteValue(hKey, pszAppPath);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

BOOL CControlPanelApp::RegisterCanonicalName(BOOL bRegister, BOOL bForAllUsers, LPCTSTR pszCanonicalName, LPCTSTR pszAppPath)
{
  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Extended Properties\\System.ApplicationName"), 
                                 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, pszAppPath, 0, REG_SZ, reinterpret_cast<const BYTE*>(pszCanonicalName), static_cast<DWORD>((_tcslen(pszCanonicalName) + 1) * sizeof(TCHAR)));
    else
      nSuccess = RegDeleteValue(hKey, pszAppPath);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

BOOL CControlPanelApp::RegisterGUID(BOOL bRegister, BOOL bForAllUsers, LPCTSTR pszGUID, LPCTSTR pszAppPath)
{
  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Extended Properties\\System.Software.AppId"), 
                                 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, pszAppPath, 0, REG_SZ, reinterpret_cast<const BYTE*>(pszGUID), static_cast<DWORD>((_tcslen(pszGUID) + 1) * sizeof(TCHAR)));
    else
      nSuccess = RegDeleteValue(hKey, pszAppPath);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

BOOL CControlPanelApp::RegisterTaskLinks(BOOL bRegister, BOOL bForAllUsers, LPCTSTR pszTaskLinksFile, LPCTSTR pszAppPath)
{
  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Extended Properties\\System.Software.TasksFileUrl"), 
                                 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, pszAppPath, 0, REG_SZ, reinterpret_cast<const BYTE*>(pszTaskLinksFile), static_cast<DWORD>((_tcslen(pszTaskLinksFile) + 1) * sizeof(TCHAR)));
    else
      nSuccess = RegDeleteValue(hKey, pszAppPath);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

BOOL CControlPanelApp::RegisterSafeMode(BOOL bRegister, BOOL bForAllUsers, DWORD dwValue, LPCTSTR pszAppPath)
{
  //Open the appropiate registry key
  HKEY hKey = NULL;
  DWORD dwDisposition = 0;
  LONG nSuccess = RegCreateKeyEx(bForAllUsers ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Extended Properties\\System.ControlPanel.EnableInSafeMode"), 
                                 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
  if (nSuccess == ERROR_SUCCESS)
  {
    //Set the value
    if (bRegister)
      nSuccess = RegSetValueEx(hKey, pszAppPath, 0, REG_DWORD, reinterpret_cast<const BYTE*>(&dwValue), sizeof(dwValue));
    else
      nSuccess = RegDeleteValue(hKey, pszAppPath);
  
    RegCloseKey(hKey);
  }

  //Check for success
  if (nSuccess == ERROR_SUCCESS)
    return TRUE;
  else
  {
    //Set the error so that calling code can determine the reason of failure
    SetLastError(nSuccess);
    return FALSE;
  }
}

CString CControlPanelApp::DoReplaceableStringParameters(const CString& sPath)
{
  //Form the values for the replacable paramater
  TCHAR szModule[_MAX_PATH];
  GetModuleFileName(AfxGetInstanceHandle(), szModule, _MAX_PATH);
  TCHAR szDrive[_MAX_DRIVE];
  TCHAR szDir[_MAX_DIR];
  TCHAR szAppPath[_MAX_PATH];
  _tsplitpath_s(szModule, szDrive, sizeof(szDrive)/sizeof(TCHAR), szDir, sizeof(szDir)/sizeof(TCHAR), NULL, 0, NULL, 0);
  _tmakepath_s(szAppPath, sizeof(szAppPath)/sizeof(TCHAR), szDrive, szDir, NULL, NULL);

  CString sAppPath(szAppPath);
  int nAppPathLength = sAppPath.GetLength();
  if (nAppPathLength && (sAppPath.GetAt(nAppPathLength - 1) == _T('\\')))
    sAppPath = sAppPath.Left(nAppPathLength - 1);


  //What will be the return value from this function
  CString sReturnPath(sPath);
  
  //Do the string replacements
  sReturnPath.Replace(_T("%MODULE%"), szModule);
  sReturnPath.Replace(_T("%APPPATH%"), sAppPath);

  return sReturnPath;
}